var Install = function()
{
	var install = {};
	var debug = 0;
	var checks;
	
	install.initPage = function()
	{
		if (!App.getActiveProjectId()) return;
		Events.addEventHandler('typeFilter', 'change', install.filterOnChange);
		Events.addEventHandler('nameFilter', 'change', install.filterOnChange);
		Events.addEventHandler('messageFilter', 'change', install.filterOnChange);

		Buttons.addClickHandler('btnFixItems', install.fixInstallation);
		Buttons.addClickHandler('btnCreateMissingItems', install.fixInstallation);
		Buttons.addClickHandler('btnReggieUpdated', install.fixInstallation);
		
		install.checkInstallation('Validate', 'Checking installation...');
	}
	
	install.fixInstallation = function()
	{
		install.checkInstallation('Install', 'Fixing installation...');
	}

	
	install.checkInstallation = function(cmd, workText)
	{
		Doc.hide('item-list');
		Doc.hide('itemTable');
		Doc.hide('createMissingItems');
		Doc.hide('fixIncompleteItems');
		Doc.hide('registerUpdated');
		
		var url = '../Install.servlet?ID=' + App.getSessionId();
		url += '&cmd='+cmd;
		Doc.addClass('workInProgress', 'working');
		Wizard.showLoadingAnimation(workText);
		Wizard.asyncJsonRequest(url, install.onChecked);
	}
	
	install.onChecked = function(response)
	{
		Doc.removeClass('workInProgress', 'working');
		
		checks = response.checks;
		
		var numMissing = 0;
		var numWarnings = 0;
		var numErrors = 0;
		var numIncomplete = 0;
		
		var index = 0;
		var topLines = []; // Top lines are for items with error/warning/special message
		var bottomLines = []; // Bottom lines are for all 'OK' items
		var allOk = true;
		var clickableItems = [];
		var typeFilter = Doc.element('typeFilter');
		typeFilter.length = 0;
		typeFilter[0] = new Option();
		var types = [];
		
		for (var checkNo = 0; checkNo < checks.length; checkNo++)
		{
			var check = checks[checkNo];
			
			var icon = 'ok.png';
			check.ok = true;
			if (check.status == 'missing') 
			{
				numMissing++;
				icon = 'error.png';
				check.ok = false;
			}
			if (check.status == 'incomplete')
			{
				numIncomplete++;
				icon = 'warning.png';
				check.ok = false;
			}
			if (check.status == 'error') 
			{
				numErrors++;
				icon = 'error.png';
				check.ok = false;
			}
			if (check.status == 'warning') 
			{
				numWarnings++;
				icon = 'warning.png';
				check.ok = false;
			}
			allOk &= check.ok;
			
			if (!types[check.itemType])
			{
				typeFilter[typeFilter.length] = new Option(check.itemType);
				types[check.itemType] = 1;
			}
			
			var line = '<tr id="check'+checkNo+'"class="highlight '+check.itemType+' ' + (check.ok ? 'check-ok' : 'check-not-ok') + '">';
			line += '<td class="indexCol"><span id="index.'+checkNo+'">'+(checkNo+1)+'</span></td>';

			line += '<td class="dottedleft">'+Strings.encodeTags(check.itemType)+'</td>';
			var name = Strings.encodeTags(check.name);
			if (check.mainType) name += ' <span class="itemsubtype">[' + check.mainType + ']</span>'; 
			if (check.id)
			{
				clickableItems[clickableItems.length] = check.itemType+check.id;
				line += '<td><div class="link" id="'+check.itemType+check.id+'" data-item-type="'+check.itemType+'" data-item-id="'+check.id+'"';
				line += ' title="View this item (use CTRL, ALT or SHIFT to edit)">'+name+'</div></td>';
			}
			else
			{
				line += '<td class="itemTypeCol"><i>' + name + '</i></td>';
			}
			line += '<td class="iconCol"><img src="../images/'+icon+'"></td>';
			line += '<td>';
			if (check.messages.length > 1)
			{
				for (var m = 0; m < check.messages.length; m++)
				{
					line += '• '+check.messages[m] + '<br>';
				}
			}
			else
			{
				line += check.messages;
			}
			if (check.stderr)
			{
				line += '<div class="stacktrace">'+Strings.encodeTags(check.stderr)+'</div>';
			}
			line += '</td></tr>';
			
			if (!check.ok || check.messages != 'Ok')
			{
				topLines[topLines.length] = line;
			}
			else
			{
				bottomLines[bottomLines.length] = line;
			}
			index++;
		}
		
		if (topLines.length + bottomLines.length > 0)
		{
			Doc.element('item-list').innerHTML = topLines.join('') + bottomLines.join('');
			Doc.show('item-list');
		}
		Doc.show('itemTable', 'table');

		// Add click handler to items
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		if (numErrors > 0)
		{
			Wizard.setWizardStatus('messagecontainer error', numErrors+' errors was detected. You need to fix those manually.');	
		}
		else if (numMissing > 0)
		{
			Doc.show('createMissingItems');
		}
		else if (numIncomplete > 0)
		{
			Doc.show('fixIncompleteItems');
		}
		else 
		{
			if (numWarnings > 0)
			{
				Wizard.setWizardStatus('all-checks-passed with-warnings', numWarnings+' warnings was detected. Reggie may still work. If not, you need to fix it manually.');
			}
			else
			{
				Wizard.setWizardStatus('all-checks-passed', 'All checks passed');
			}
			if (response.installedVersion != Reggie.VERSION)
			{
				Doc.show('registerUpdated');
			}
		}
	}
	
	install.filterOnChange = function()
	{
		
		var filter = new Filter();
		filter.addTextCriteria(ItemProperty.ItemType, Doc.element('typeFilter').value);
		filter.addContainsCriteria(ItemProperty.Name, Doc.element('nameFilter').value);
		filter.addContainsCriteria(ItemProperty.Messages, Doc.element('messageFilter').value);

		var numMatch = 0;
		for (var checkNo = 0; checkNo < checks.length; checkNo++)
		{
			var check = checks[checkNo];
			
			check.include = filter.check(check);
			if (check.include)
			{
				numMatch++;
				Doc.show('check'+checkNo);
			}
			else
			{
				Doc.hide('check'+checkNo);
			}
		}
		
		Doc.showHide('item-list', numMatch > 0);
		Doc.showHide('no-matching-items', numMatch==0);
	}

	
	return install;
}();

Doc.onLoad(Install.initPage);

var ItemProperty = function()
{
	var p = {};
	
	p.Name = function(item)
	{
		return item.name;
	}
	
	p.ItemType = function(item)
	{
		return item.itemType;
	}

	p.Messages = function(item)
	{
		return item.messages.join('');
	}
	
	return p;
}();

/**
	Holds one or more filter criterias. Use the 'add*' methods
	to add criterias to the filter, and the 'check(item)' method
	to evaluate all criterias against a single item.
*/
var Filter = function()
{
	this.criterias = [];
}

/**
	Adds a text criteria comparing the given item property to
	a static value (f2).
*/
Filter.prototype.addTextCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.eq;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a text criteria checking if the given item property to
	contains a static value (f2).
*/
Filter.prototype.addContainsCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.contains;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Evaluate all criterias. Return true if all pass, false otherwise.
*/
Filter.prototype.check = function(item)
{
	for (var i = 0; i < this.criterias.length; i++)
	{
		if (!this.criterias[i](item)) return false;
	}
	return true;
}


var Comparator = function()
{
	var cmp = {};
	
	/**
		Return a comparator based on the symbol
		which can be either '>' or '<' to return
		Comparator.gt or Comparator.lt. Otherwise
		the given default or Comparator.eq is returned.
	*/
	cmp.get = function(symbol, defaultCmp)
	{
		var c;
		if (symbol == '<')
		{
			c = cmp.lt;
		}
		else if (symbol == '>')
		{
			c = cmp.gt;
		}
		else
		{
			c = defaultCmp || cmp.eq;
		}
		return c;
	}
	
	// Compare if two values are equal
	cmp.eq = function(v1, v2)
	{
		return v1 == v2;
	}
	
	// Compare if two values are different
	cmp.neq = function(v1, v2)
	{
		return v1 != v2;
	}
	
	// Check if v2 is contained in v1
	cmp.contains = function(v1, v2)
	{
		return v1 != null && v2 != null && v1.indexOf(v2) >= 0;
	}
	
	// Compare if two values are appoximately equals (difference is not more than 0.05)
	cmp.approx = function(v1, v2)
	{
		return Math.abs(v2 - v1) < 0.05;
	}
	
	// Compare if v1 starts with the same string as v2
	cmp.startsWith = function(v1, v2)
	{
		return v1 != null && v1.indexOf(v2) == 0;
	}
	
	// Compare if v1 is greater than or equal to v2
	cmp.gt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 >= v2;
	}
	
	// Compare if v1 is less than or equal to v2
	cmp.lt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 <= v2;
	}
	
	return cmp;
}();



