var GenoType = function()
{
	var gt = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeAlignedSequences = null;
	var annotationTypePipeline = null;
	var alignedSequences;
	var manuallySelected = [];
	
	// Page initialization
	gt.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectAlignedSequences', gt.selectAlignedSequences);
		Events.addEventHandler('alignedSequences', 'base-selected', gt.setAlignedSequenceCallback);
		Events.addEventHandler('alignedSequences', 'change', gt.alignedSequencesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', gt.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', gt.initializeStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', gt.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetAlignedSequencesForGenotypeQc';
		Wizard.asyncJsonRequest(url, gt.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	gt.initializeStep1 = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		var msg = response.existingAlignmentsCount + ' alignments';
		if (response.existingGenotypeCallsCount) msg += ' and '+response.existingGenotypeCallsCount+' genotype calls';
		Doc.element('alreadyChecked').innerHTML = msg;
		
		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var firstPlate = null;
			var numNoPatient = 0;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var seq = alignedSequences[asNo];
				var lib = seq.lib;
				var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;

				if (!seq.patient) numNoPatient++;
				
				var option = gt.createListOption(asNo+1, seq, seq.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate);
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
			
			if (numNoPatient > 0)
			{
				Wizard.showGoNextConfirmation(true, 'Some alignments have no patient information and will be excluded from the genotype check.');
			}
			Events.sendChangeEvent('alignedSequences');
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'No new alignments available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.alignedSequences.focus();
	}
	
	gt.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	gt.selectAlignedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.alignedSequences.disabled) return;
		
		if (subtypeAlignedSequences == null)
		{
			subtypeAlignedSequences = Reggie.getSubtypeInfo('ALIGNED_SEQUENCES');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeAlignedSequences.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent('RNAseq/Hisat/StringTie');
		Dialogs.selectItem('DERIVEDBIOASSAY', 'alignedSequences', 1, url);
	}

	gt.setAlignedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var opt = Reggie.getListOption('alignedSequences', ms.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = ms.id;
		}
		
		if (ms.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../Genotype.servlet?ID='+App.getSessionId();
			url += '&cmd=GetAlignedSequencesForGenotypeQc';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, gt.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('alignedSequences');
		}

	}
	gt.manuallySelected = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var offset = frm.alignedSequences.length+1;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var as = alignedSequences[asNo];
				var option = gt.createListOption(asNo+offset, as, as.DO_NOT_USE == null);
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
			Events.sendChangeEvent('alignedSequences');
		}
	}

	gt.createListOption = function(index, alignedItem, selected)
	{
		var lib = alignedItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += alignedItem.name;
		if (!alignedItem.patient) name += ' (no patient data)';
		
		var option = new Option(name, alignedItem.id, false, selected && alignedItem.patient);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		if (!alignedItem.patient) option.disabled = true;
		option.alignedSequences = alignedItem;
		return option;
				
		
//		var name = (asNo+1) + ': ';
//		if (libPlate) name += libPlate.name + ' - ';
//		name += seq.name;
		/*
		if (!seq.patient) 
		{
			name += ' (no patient data)';
			numNoPatient++;
		}
		
		var option = new Option(name, seq.id, false, libPlate != null && libPlate.name==firstPlate && seq.patient);
		if (!seq.patient) 
		{
			option.disabled = true;
		}
		else if (isYellow) 
		{
			option.className = 'yellow';
		}
		option.alignedSequences = seq;
*/
		
	}

	
	gt.alignedSequencesOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;
		Wizard.setInputStatus('alignedSequences');
		Wizard.hideGoNextConfirmation();

		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				numSelected++;
				var as = frm.alignedSequences[asNo].alignedSequences;
				if (as.DO_NOT_USE) numDoNotUse++;
				if (as.pipeline && as.pipeline != 'RNAseq/Hisat/StringTie')
				{
					invalidPipeline = Strings.encodeTags(as.name+' ('+as.pipeline)+') is not intended for the RNAseq/Hisat/StringTie pipeline.';
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('alignedSequences', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'valid');
		}
	}

	gt.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		var alignedSequences = [];
		submitInfo.alignedSequences = alignedSequences;
			
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				var as = {};
				as.id = frm.alignedSequences[asNo].alignedSequences.id;
				alignedSequences[alignedSequences.length] = as;
			}
		}

		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GenotypeQcCheck';
		Wizard.showLoadingAnimation('Comparing genotypes...', 'genotypes-progress');
		Wizard.asyncJsonRequest(url, gt.checkCompleted, 'POST', JSON.stringify(submitInfo));
	}
	
	gt.checkCompleted = function(response)
	{
		alignedSequences = response.alignedSequences;
		
		var html = '<table id="compareTable">';
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th colspan="3">Comparisons</th>';
		html += '<th class="dottedleft"></th>';
		html += '<th colspan="2">ALIGNED_PAIRS</th>';
		html += '<th  class="dottedleft" colspan="3">Genotypes</th>';
		html += '<th class="dottedleft" colspan="2"></th>';
		html += '<th class="dottedleft" colspan="4"></th>';
		html += '<th class="dottedleft" colspan="2">Actions</th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '<tr>';
		html += '<th>Total</th>';
		html += '<th>Same PAT</th>';
		html += '<th>Skipped</th>';
		html += '<th class="dottedleft">Libplate</th>';
		html += '<th>(millions)</th>';
		html += '<th>Dup.</th>';
		html += '<th class="dottedleft">Total</th>';
		html += '<th>HET</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft">Alignment</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft icon-col"></th>';
		html += '<th colspan="2">Warnings and messages</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft"><span id="disable" data-prefix="disable" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Disable</span></th>';
		html += '<th><span id="flagrna" data-prefix="flag" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Flag</span></th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		html += '</thead>';
		
		var htmlNoMessages = '';
		var htmlNoWarnings = '';
		
		for (var aNo = 0; aNo < alignedSequences.length; aNo++)
		{
			var aligned = alignedSequences[aNo];
			var lib = aligned.lib;
			var cmp = aligned.compare;
			var vcf = cmp.vcf;
			
			var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
			var alignedReads = aligned.ALIGNED_PAIRS == null ? '-' : Reggie.formatNumber(aligned.ALIGNED_PAIRS/1000000, null, 2);
			var duplication = aligned.FRACTION_DUPLICATION == null ? '-' : Reggie.formatNumber(100*aligned.FRACTION_DUPLICATION, '%', -1);
			var hetPercentage = vcf.gtCount ? Reggie.formatNumber(100*vcf.hetCount/vcf.gtCount, '%', -1) : '-';

			var disableChecked = cmp.recommendDisable ? "checked" : "";
			var flagChecked = cmp.recommendFlag ? "checked" : "";
			
			var rowspan = cmp.messages.length || 1;
			
			var tmp = '<tbody class="highlight"><tr>';
			tmp += '<td rowspan="'+rowspan+'">'+cmp.total+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+(cmp.samePat || '')+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+(cmp.skipped || '')+'</td>';
			tmp += '<td rowspan="'+rowspan+'" class="dottedleft text-col">'+(libPlate ? Strings.encodeTags(libPlate.name + ' ' + lib.bioWell.location) : '-')+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+alignedReads+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+duplication+'</td>';
			tmp += '<td rowspan="'+rowspan+'" class="dottedleft">'+vcf.gtCount+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+hetPercentage+'</td>';
			tmp += '<td rowspan="'+rowspan+'" class="icon-col">';
			tmp += '<span class="link vcf-link" data-file-id="'+vcf.fileId+'" data-item-id="'+aligned.id+'" title="View genotype statistics"><img src="../images/vcf_file.png"></span>';
			tmp += '</td>';
			tmp += '<td rowspan="'+rowspan+'" class="dottedleft text-col">'+Strings.encodeTags(aligned.name)+'</td>';
			tmp += '<td rowspan="'+rowspan+'" class="icon-col"><span class="link case-summary" data-name="'+Strings.encodeTags(aligned.name)+'" title="Show case summary"><img src="../images/case_summary.png"></span></td>';

			// The first message is on the main <tr>
			var numWarnings = 0;
			if (cmp.messages.length > 0)
			{
				var msg = cmp.messages[0];
				if (msg.warningLevel != 'INFO') numWarnings++;
				tmp += gt.getMessageHtml(msg, aligned, vcf);
			}
			else
			{
				tmp += '<td class="dottedleft icon-col"></td>';
				tmp += '<td class="text-col" colspan="2"></td>';
				tmp += '<td class="icon-col"></td>';
			}
			tmp += '<td rowspan="'+rowspan+'" class="dottedleft"><input type="checkbox" name="disable.'+aligned.id+'" '+disableChecked+'></td>';
			tmp += '<td rowspan="'+rowspan+'"><input type="checkbox" name="flag.'+aligned.id+'" '+flagChecked+'></td>';
			tmp += '<td rowspan="'+rowspan+'" class="dottedleft comment"><input type="text" name="comment.'+aligned.id+'" value="'+Strings.encodeTags(aligned.QC_GENOTYPE_COMMENT)+'"></td>';
			tmp += '</tr>';
			
			// If there are more messages, add more <tr>
			for (var mNo = 1; mNo < cmp.messages.length; mNo++)
			{
				var msg = cmp.messages[mNo];
				if (msg.warningLevel != 'INFO') numWarnings++;
				tmp += '<tr>'+gt.getMessageHtml(msg, aligned, vcf) + '</tr>';
			}
			tmp += '</tbody>';
			
			if (cmp.messages.length == 0)
			{
				// No messages are sorted last in the list
				htmlNoMessages += tmp;
			}
			else if (numWarnings == 0)
			{
				// No warnings are sorted in the middle
				htmlNoWarnings += tmp;
			}
			else
			{
				// Items with at least one warning are sorted first
				html += tmp;
			}
		}
		
		html += htmlNoWarnings;
		html += htmlNoMessages;
		html += '</table>';
		Doc.element('comparisons').innerHTML = html;

		Events.addEventHandler('flagrna', 'click', gt.toggleSelection);
		Events.addEventHandler('disable', 'click', gt.toggleSelection);

		// Add click handler to VCF files
		var clickableItems = document.getElementsByClassName('vcf-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', gt.vcfLinkOnClick);
		}
		
		// Case summary links
		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}

		Wizard.setCurrentStep(2);
		Doc.show('step-2');
		Doc.show('goregister');
		Doc.show('gocancel');
	}

	
	gt.getMessageHtml = function(message, aligned, vcf)
	{
		var level = message.warningLevel;
		var icon = null;
		if (level == 'INFO')
		{
			icon = 'ok.png';
		}
		else if (level == 'LOW')
		{
			icon = 'warning_small.png';
		}
		else
		{
			icon = 'warning.png';
		}

		var html = '';
		html += '<td class="dottedleft icon-col"><img src="../images/'+icon+'"></td>';
		html += '<td class="text-col" title="'+(message.messageTooltip || '')+'">'+Strings.encodeTags(message.message) + '</td>';
		html += '<td class="text-col">';
		if (message.otherAssay)
		{
			html += gt.tagCommonPartOfName(aligned.name, message.otherAssay);
			if (message.DO_NOT_USE) html += '<img src="../images/donotuse.png" title="DoNotUse-'+Strings.encodeTags(message.DO_NOT_USE + ': ' + message.DO_NOT_USE_COMMENT)+'">';
			if (message.flagged) html += '<img src="../images/flag.png" title="'+Strings.encodeTags(message.otherAssay)+' is already flagged">';
			if (message.currentVerifiedBy) 
			{
				var vicon = message.currentVerifiedBy == 'LYSATE' ? 'gt-verified-lysate.png' : 'gt-verified.png';
				html += '<img src="../images/'+vicon+'" title="'+Strings.encodeTags(message.otherAssay+' is already verified by '+message.currentVerifiedBy)+'">';
			}
		}
		html += '</td>';
		html += '<td class="icon-col">';
		if (message.vcf)
		{
			html += ' <span class="link vcf-link" data-file-id="'+vcf.fileId+'" data-item-id="'+aligned.id+'"';
			html += ' data-item-id2="'+message.id+'" data-file-id2="'+message.vcf.fileId+'"';
			html += ' title="Compare genotypes (hold CTRL, ALT or SHIFT to open in a new popup)"><img src="../images/vcf_file.png"></span>';
		}
		html += '</td>';
		return html;
	}
	
	
	gt.tagCommonPartOfName = function(name1, name2)
	{
		var n1 = name1.split('.');
		var n2 = name2.split('.');
		var i = 0;
		
		var common = '';
		var sep = '.';
		while (i < n1.length && i < n2.length && n1[i] == n2[i])
		{
			common += n1[i] + sep;
			i++;
		}
		
		var diff = '';
		sep = '';
		while (i < n2.length)
		{
			diff += sep + n2[i];
			i++;
			sep = '.';
		}
		
		return '<span class="commonpart">'+Strings.encodeTags(common)+'</span><span class="diffpart">'+Strings.encodeTags(diff)+'</span>';
	}
	
	gt.vcfLinkOnClick = function(event)
	{
		var fileId = Data.int(event.currentTarget, 'file-id');
		var itemId = Data.get(event.currentTarget, 'item-id');
		var fileId2 = Data.int(event.currentTarget, 'file-id2');
		var itemId2 = Data.get(event.currentTarget, 'item-id2');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var url = 'view_genotypes.jsp?ID=' + App.getSessionId();
		url += '&fileId='+fileId;
		url += '&itemId='+itemId;
		url += '&fileId2='+fileId2;
		url += '&itemId2='+itemId2;
		
		Dialogs.openPopup(url, (specialKey ? 'ViewVcf'+itemId+itemId2 : 'ViewVcf'), 900, 600);
	}
	
	gt.toggleSelection = function(event)
	{
		var prefix = Data.get(event.currentTarget, 'prefix');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var frm = document.forms['reggie'];
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];
			var chk = frm[prefix+'.'+aligned.id];
			if (chk && !chk.disabled)
			{
				chk.checked = specialKey ? false : !chk.checked;
			}
		}
	}

	gt.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		var list = [];
		submitInfo.alignedSequences = list;
		
		for (var aNo = 0; aNo < alignedSequences.length; aNo++)
		{
			var aligned = alignedSequences[aNo];
			var cmp = aligned.compare;
				
			var tmp = {};
			tmp.id = aligned.id;
			tmp.disable = frm['disable.'+aligned.id].checked;
			if (frm['flag.'+aligned.id].checked)
			{
				var flag = [];
				tmp.flag = flag;
				flag[flag.length] = aligned.id;
			}
			var verified = [];
			tmp.verified = verified;
			var flag = null;
			if (frm['flag.'+aligned.id].checked)
			{
				flag = [];
				tmp.flag = flag;
				flag[flag.length] = aligned.id;
			}
			for (var mNo = 0; mNo < cmp.messages.length; mNo++)
			{
				var msg = cmp.messages[mNo];
				if (msg.id)
				{
					if (flag && msg.assayType == 'ALIGNMENT' && msg.warningLevel != 'INFO')
					{
						flag[flag.length] = msg.id;
					}
					if (msg.verifiedBy)
					{
						verified[verified.length] = { 'id': msg.id, 'verifiedBy': msg.verifiedBy };
					}
				}
			}
			
			tmp.comment = frm['comment.'+aligned.id].value;
			list[list.length] = tmp;
		}
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterQcCheck';
		Wizard.showLoadingAnimation('Registering...');
		Wizard.asyncJsonRequest(url, gt.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	gt.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return gt;
}();

Doc.onLoad(GenoType.initPage);

