var StringTie = function()
{
	var stringtie = {};
	var debug = 0;
	
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};
	var rawBioAssays;
	
	// Page initialization
	stringtie.initPage = function()
	{
		
		// Step 1

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', stringtie.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../StringTie.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmed';
		Wizard.asyncJsonRequest(url, stringtie.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	stringtie.initializeStep1 = function(response)
	{
		rawBioAssays = response.rawBioAssays;
		if (rawBioAssays.length == 0)
		{
			Wizard.setFatalError('No raw bioassays available for processing.');
			return;
		}
		
		var html = '<table id="rawBioAssaysTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="3">StringTie</th>'
		html += '<th class="dottedleft" colspan="3">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		
		html += '<tr>';
		html += '<th>Library</th>';
		html += '<th class="dottedleft">Plate</th>';
		html += '<th class="dottedleft">Job</th>'
		html += '<th>gene.tsv</th>';
		html += '<th>Files</th>';
		html += '<th class="dottedleft"><span id="flagrna" data-prefix="flag" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Flag RNA</span></th>';
		html += '<th><span id="rerun" data-prefix="rerun" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Re-run</span></th>';
		html += '<th><span id="ssp" data-prefix="ssp" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">SSP analysis</span></th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		var numWithError = 0;
		var yellowImg = '<img src="../images/yellow-label.png">';
		for (var rawNo = 0; rawNo < rawBioAssays.length; rawNo++)
		{
			var raw = rawBioAssays[rawNo];

			var lib = raw.lib;
			lib.stratagene = Reggie.isStratagene(lib.name);
			lib.external = Reggie.isExternal(lib.name);
			var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
			var img = isYellow ? yellowImg : '';
			var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
			
			var stringTieJob = raw.stringTieJob;
			var isError = !raw.geneTsvFile || !raw.spots;
			var flagChecked = false;
	
			html += '<tr class="highlight ' + (rawNo % 4 < 2 ? 'evenrow' : 'oddrow') + (isYellow ? ' yellow-specimen' : '') + '">';
			html += '<td class="prompt if-yellow">'+img+Strings.encodeTags(raw.name)+'</td>';
			html += '<td class="dottedleft">'+(libPlate ? Strings.encodeTags(libPlate.name) : '-') +'</td>';

			html += '<td class="dottedleft">';
			if (stringTieJob)
			{
				isError |= stringTieJob.status == 'ERROR';
				var image = JOB_IMAGE[stringTieJob.status];
				if (!isError && !raw.geneTsvFile) image = 'warning.png';
				html += '<span class="link item-link" data-item-type="JOB" data-item-id="'+stringTieJob.id+'" title="'+Strings.encodeTags(stringTieJob.statusMessage)+'"><img src="../images/'+image+'"></span>';
			}
			html += '</td>';
			html += '<td>';
			html += raw.geneTsvFile ? raw.spots : '-';
			html += '</td>';

			if (raw.DataFilesFolder && stringTieJob && stringTieJob.server)
			{
				html += '<td>';
				html += '<span class="link file-link" data-server="'+Strings.encodeTags(stringTieJob.server)+'" data-folder="'+Strings.encodeTags(raw.DataFilesFolder)+'" data-item-id="'+raw.id+'" title="View generated files"><img src="../images/remote_folder.png"></span>';
				html += '</td>';
			}
			else
			{
				html += '<td>-</td>';
			}
			// Do not allow flagging external or stratagene
			var sspChecked = !isError ? ' checked' : '';
			var reRunChecked = isError ? ' checked' : '';
			var sspDisabled = isError ? ' disabled' : '';
			if (isError) numWithError++;
			
			if (lib.stratagene || lib.external)
			{
				html += '<td class="dottedleft">-</td>';
			}
			else
			{
				html += '<td class="dottedleft"><input type="checkbox" name="flag.'+raw.id+'"'+(flagChecked ? ' checked':'')+'></td>';
			}
			
			html += '<td><input type="checkbox" name="rerun.'+raw.id+'"'+reRunChecked+'></td>';
			html += '<td><input type="checkbox" name="ssp.'+raw.id+'"'+sspChecked+sspDisabled+'></td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+raw.id+'"></td>';
			html += '</tr>';

		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('rawBioAssays').innerHTML = html;
		
		Events.addEventHandler('flagrna', 'click', stringtie.toggleSelection);
		Events.addEventHandler('rerun', 'click', stringtie.toggleSelection);
		Events.addEventHandler('ssp', 'click', stringtie.toggleSelection);

		// Add click handler to items
		var clickableItems = document.getElementsByClassName('item-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('file-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', stringtie.fileLinkOnClick);
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	stringtie.fileLinkOnClick = function(event)
	{
		var folder = Data.get(event.currentTarget, 'folder');
		var server = Data.get(event.currentTarget, 'server');
		var itemId = Data.get(event.currentTarget, 'item-id');
		server = server.replace(/\s*\[.*\]/, '');
		var url = 'view_remote_files.jsp?ID=' + App.getSessionId();
		url += '&clusterId='+server;
		url += '&path='+encodeURIComponent(folder);
		url += '&itemType=RAWBIOASSAY&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewProjectFiles', 750, 500);
	}
	
	stringtie.toggleSelection = function(event)
	{
		var prefix = Data.get(event.currentTarget, 'prefix');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var frm = document.forms['reggie'];
		for (var rawNo = 0; rawNo < rawBioAssays.length; rawNo++)
		{
			var raw = rawBioAssays[rawNo];
			var chk = frm[prefix+'.'+raw.id];
			if (chk && !chk.disabled)
			{
				chk.checked = specialKey ? false : !chk.checked;
			}
		}
	}

	stringtie.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.deleteItemsCreatedByFailedJobs = frm.deleteItemsCreatedByFailedJobs.checked;
		
		var all = [];
		submitInfo.rawBioAssays = all;
		for (var rawNo = 0; rawNo < rawBioAssays.length; rawNo++)
		{
			var raw = rawBioAssays[rawNo];
			var tmp = {};
			tmp.id = raw.id;
			
			tmp.flag = frm['flag.'+raw.id] && frm['flag.'+raw.id].checked;
			tmp.rerun = frm['rerun.'+raw.id].checked;
			tmp.ssp = frm['ssp.'+raw.id].checked;
			tmp.comment = frm['comment.'+raw.id].value;
			
			all[all.length] = tmp;
		}
		
		var url = '../StringTie.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterStringTie';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, stringtie.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	stringtie.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return stringtie;
}();

Doc.onLoad(StringTie.initPage);

